"""
This is the main module of the plugin. Here all the GUI elements inside the
frame are created and the connection between the tabs is made possible. There
are two operating modes: stand-alone and connected with Anybody.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Created on 7th of February, 15:16, 2017

"""
import os
import re
import shutil
import wx
import wx.grid

from ams_service.thriftapi import Connection

from bm_config import iniparse
from bm_config import history
from bm_config import address_tab
from bm_config import advtab
from bm_config import bodytab
from bm_config import legstab
from bm_config import armstab
from bm_config import trunktab
from bm_config import abouttab
from bm_config import driverstab
from bm_config import anyparse
from bm_config import bm_param_parser
from bm_config import memento
from bm_config import res


class Arguments:
    def __init__(self):
        self.any_file = None
        self.statements = None
        self.statements_dictionary = None
        self.options = None
        self.options_dictionary = None
        self.xml_file = None
        self.memento = None


class Frame(wx.Frame):
    def __init__(self, parent, title, args, config_file):
        super(Frame, self).__init__(
            parent,
            title=title,
            size=(700, 1000),
            style=(wx.DEFAULT_FRAME_STYLE ^ wx.RESIZE_BORDER),
        )

        self._arg_config_file = config_file
        self._any_file = None
        self._default_any_file = None
        self._bm_param_file = None
        self._connected = False
        self._closed = False
        self._args = None
        self._connection = None
        self._main_file = None
        self._anyfolder = None
        self._error_loading = False
        self._disabled = False
        self._status = {
            "Colour0": "LIGHT_GRAY",
            "Text0": res.OKSTAT,
            "Colour1": "LIGHT_GRAY",
            "Text1": res.PROMPTCONF,
        }
        self.statusbar = None

        script_file = self._get_script_file(args)
        self._script_file = script_file
        destroyed = False
        if script_file is None and args["port"] is not None:
            lines = self._human()
            answer = True
            interrogation = True
            if not len(lines):
                answer = self._interrogation(
                    res.NO_HUMAN, "HumanModel is not included in .main"
                )
                interrogation = False

            if not answer:
                self.Destroy()
                destroyed = True
            else:
                has_line, script_file = self._has_include_line()
                if not has_line:
                    text = res.NO_IN_FILE_TEXT
                    title = res.NO_IN_FILE_TITLE
                    answer = False
                    if interrogation:
                        answer = self._interrogation(text, title)
                    if answer:
                        self._include_input_file()
                    else:
                        openFileDialog = wx.FileDialog(
                            self,
                            "Open",
                            "",
                            "",
                            "AMS files (*.any)|*.any",
                            wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
                        )
                        openFileDialog.ShowModal()
                        script_file = openFileDialog.GetPath()
                        openFileDialog.Destroy()
                if script_file == "":
                    self.Destroy()
                    destroyed = True
                else:
                    self._make_input_file(script_file)

        if not destroyed:
            self._create_gui_elements()
        self._refresh_status()

    def _get_script_file(self, args):

        if args["port"] is not None:
            self._connection = Connection(args["port"], args["identification"])
            self._main_file = self._connection.get_main_file
            self._connected = self._connection.is_connected()
            title_name = "Main.HumanModel.Config.BMPlugin.DisplayName"
            title = self._connection.get_data(title_name)
            self.SetTitle(title)
            title = title.split("HumanModel")[0]
            title = title.replace("Main", "")
            self._anyfolder = title.strip(".")
            if self._connected:
                ams_paths = self._connection.get_paths()
                ams_defines = self._connection.get_defines()
                if "ANYBODY_PATH_BODY" in ams_paths:
                    self._bm_param_file = os.path.join(
                        ams_paths["ANYBODY_PATH_BODY"], res.BM_PARAM_FILE
                    )
                if "ANYBODY_PATH_AMMR" in ams_paths:
                    self._default_any_file = os.path.join(
                        ams_paths["ANYBODY_PATH_AMMR"],
                        res.DEFAULT_ANY_PATH,
                        res.DEFAULT_ANY_FILE,
                    )
                if "BM_CONFIG_PLUGIN" in ams_defines:
                    return ams_defines["BM_CONFIG_PLUGIN"]
                else:
                    return None
        elif self._arg_config_file is not None:
            return self._arg_config_file
        else:
            return None

    def _human(self):
        main_file = self._connection.get_main_file
        with open(main_file) as f:
            content = f.readlines()
        line_count = 0
        lines = []
        for line in content:
            if res.HUMAN_FILE in line and res.INCLUDE in line:
                lines.append(line_count)
            line_count += 1

        return lines

    def _has_include_line(self):
        main_file = self._connection.get_main_file
        if self._anyfolder == "":
            script_file = res.IN_FILE
            ex = r'(\s*|\n*)#include\s*"Model/' + res.IN_FILE + '"\n'
            regex = re.compile(ex)
        else:
            script_file = self._anyfolder + "_" + res.IN_FILE
            ex = r'(\s*|\n*)#include\s*"Model/' + script_file + '"\n'
            regex = re.compile(ex)
        with open(main_file) as f:
            content = f.read()

        if regex.search(content) is None:
            return False, script_file
        else:
            return True, script_file

    def _make_input_file(self, file):
        if not os.path.exists(file):
            main_dir = self._connection.get_configuration_folder() + "\\"
            self._script_file = main_dir + "Model\\" + file
            if not os.path.exists(self._script_file):
                f = open(self._script_file, "w")
                f.close()
        else:
            self._script_file = file
            f = open(self._script_file, "w")
            f.close()

    def _include_input_file(self):
        """This method is making use of regular expressions in order to
        add the include line for the configuration file inside the
        .main.any file. To achieve this, the regular expressions compiler
        is used in order to find check if the file is already included.
        If the file is not included, the compiler is used to determine
        if the model contains two or more human models. This is possible
        by making use of the name of the object containing the Human Model
        for which the plugin was started. When using multiple human
        models, they are declared inside an AnyFolder. So the compiler
        is looking for that particular AnyFolder instantiation which
        contains the afferent human model in order to add the include
        line there. If only one human model is used, the include line will
        be added in the first lines of the Main function.
        """

        self._connection.save_main_file()
        # The main file is saved through ams service to prevent losing user
        #   changes.

        main_file = self._connection.get_main_file
        # The address of the main file is obtained from ams service.
        # EXAMPLE:
        #   C:\Users\ms\Documents\StandingModel\StandingModel.main.any

        print("main_file", main_file)

        with open(main_file) as f:
            content = f.read()

        regex = re.compile(
            r"(\s*|\n*)AnyFolder(\s*|\n*)" + self._anyfolder + r"(\s*|\n*)=(\s*|\n*){"
        )
        # The compiler is looking the general expression:
        #   "AnyFolder <FolderName> = {", where <FolderName> is the name of
        #   AnyFolder object containing the human model for which the plugin
        #   was started.
        # EXAMPLE:
        #   AnyFolder H2 = {
        #   'H2' is the name of the object containing the human model.

        match = regex.search(content)
        # The search returns None if no match for the given general expression
        #   was found.

        if match is not None:
            match = match.group()
            head, tail = content.split(match)
            file = "Model/" + self._anyfolder + "_" + res.IN_FILE
            include_line = match + '\n    #include "' + file + '"\n'
            # The include line for the configuration file is put together.
            # EXAMPLE:
            #   #include "Model/H2_BodyModelConfiguration.any"

            content = head + include_line + tail
        else:
            regex = re.compile(r"(\s*|\n*)Main(\s*|\n*)=(\s*|\n*){")
            match = regex.search(content).group()
            # The compiler is looking the general expression:
            #   "Main = {"

            head, tail = content.split(match)
            file = "Model/" + res.IN_FILE
            include_line = match + '\n  #include "' + file + '"\n'
            # The include line for the configuration file is put together.
            # EXAMPLE:
            #   #include "Model/BodyModelConfiguration.any"

            content = head + include_line + tail

        with open(main_file, "w") as f:
            f.write(content)

        self._connection.reload_main_file()

    def _interrogation(self, question, caption="Yes or no?"):
        dlg = wx.MessageDialog(self, question, caption, wx.YES_NO | wx.ICON_QUESTION)
        result = dlg.ShowModal() == wx.ID_YES
        dlg.Destroy()
        return result

    def _create_gui_elements(self):
        if self._bm_param_file is None:
            self._xml_address = res.bm_param_address
        else:
            self._xml_address = self._bm_param_file

        self._appdata_file = iniparse.INIParse(res.appdata_address, "config_data")
        self.SetIcon(wx.Icon(res.icon_address, wx.BITMAP_TYPE_ICO))
        #
        self._adv_tab = None
        self._focus = 0

        self.Bind(wx.EVT_CLOSE, self._on_exit)
        # define the notebook gui element
        self._panel = wx.Panel(self)
        self._tabs = wx.Notebook(self._panel)
        #

        # make sizers for the button area and the sizer which will contain the
        # notebook and the buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        frame_sizer = wx.BoxSizer(wx.VERTICAL)
        #

        menubar = wx.MenuBar()
        fileMenu = wx.Menu()
        fileItem = fileMenu.Append(wx.ID_EXIT, "Exit\tAlt+F4", "Exit Plugin")
        menubar.Append(fileMenu, "&File")

        editMenu = wx.Menu()
        undoItem = editMenu.Append(wx.ID_UNDO, "Undo\tCtrl+Z", "Undo last change")
        redoItem = editMenu.Append(wx.ID_REDO, "Redo\tCtrl+Y", "Redo change")
        editMenu = menubar.Append(editMenu, "&Edit")

        self.SetMenuBar(menubar)

        self.Bind(wx.EVT_MENU_OPEN, self._on_edit)
        self.Bind(wx.EVT_MENU, self._on_exit, fileItem)
        self.Bind(wx.EVT_MENU, self._on_undo, undoItem)
        self.Bind(wx.EVT_MENU, self._on_redo, redoItem)

        undo_id, redo_id, exit_id = wx.NewId(), wx.NewId(), wx.NewId()
        self.Bind(wx.EVT_MENU, self._on_undo, id=undo_id)
        self.Bind(wx.EVT_MENU, self._on_redo, id=redo_id)
        self.Bind(wx.EVT_MENU, self._on_exit, id=exit_id)

        acc_tbl = wx.AcceleratorTable(
            [
                (wx.ACCEL_CTRL, ord("Z"), undo_id),
                (wx.ACCEL_CTRL, ord("Y"), redo_id),
                (wx.ACCEL_ALT, wx.WXK_F4, exit_id),
            ]
        )
        self.SetAcceleratorTable(acc_tbl)
        undoItem.Enable(False)
        redoItem.Enable(False)
        self._undo = undoItem
        self._redo = redoItem

        # initialize the address tab
        self._adrs_tab = address_tab.AddressTab(self._tabs)
        self._tabs.AddPage(self._adrs_tab, res.ADRS_TAB)
        self._about_tab = abouttab.AboutTab(self._tabs)
        self._tabs.AddPage(self._about_tab, res.ABOUT_TAB)

        #

        # buttons and history drop-down define
        self._ok_button = wx.Button(self._panel, wx.ID_OK)
        self._ok_button.Disable()

        CLOSE_button = wx.Button(self._panel, wx.ID_CANCEL)
        self._default_button = wx.Button(self._panel, label=res.BL_DEFAULT)
        self._default_button.Disable()

        self._history = wx.Choice(self._panel, size=(170, 23))
        self._history.Disable()
        Text = wx.StaticText(self._panel, label=res.L_HISTORY, size=(80, 20))
        #

        # layout management through sizers
        button_sizer.Add(self._default_button, 0, wx.ALIGN_LEFT, 3)
        button_sizer.Add(
            Text, 0,  wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 3
        )
        button_sizer.Add(self._history, 1, wx.ALIGN_CENTER_VERTICAL, 3)
        if self._connected:
            self._load_button = wx.Button(self._panel, wx.ID_ANY, label=res.BL_LOAD)
            button_sizer.Add(self._load_button, 1, wx.LEFT, 3)
            self.Bind(wx.EVT_BUTTON, self._on_load, id=self._load_button.GetId())
            self._load_button.Disable()

        button_sizer.Add(self._ok_button, 1, wx.LEFT, 3)
        button_sizer.Add(CLOSE_button, 1, wx.LEFT, 3)

        frame_sizer.Add(self._tabs, 1, wx.EXPAND | wx.TOP | wx.LEFT, 3)
        frame_sizer.Add(button_sizer, 0, wx.LEFT | wx.RIGHT, 3)

        self._panel.SetSizer(frame_sizer)
        #

        # gui bindings to event handlers
        self.Bind(wx.EVT_BUTTON, self._on_ok, id=self._ok_button.GetId())
        self.Bind(wx.EVT_BUTTON, self._on_exit, id=CLOSE_button.GetId())
        self.Bind(wx.EVT_BUTTON, self._on_default, id=self._default_button.GetId())
        self.Bind(wx.EVT_CHOICE, self._on_choice, id=self._history.GetId())
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGING, self._on_change, id=self._tabs.GetId())
        self.Bind(wx.EVT_ACTIVATE, self._on_focus, id=self.GetId())
        #

        self.statusbar = self.CreateStatusBar(2)
        self.statusbar.SetStatusWidths([100, 600])
        self.statusbar.SetStatusText(res.OKSTAT, 0)
        self.statusbar.SetStatusText(res.PROMPTANY, 1)

        self._panel.Show()
        frame_sizer.Fit(self)
        self.SetSize(700, 660)
        if self._script_file is not None:
            self.create_remaining_tabs(self._script_file)

        self.Show()
        self._refresh_status()

    def _on_edit(self, event):
        undo, redo = self._memento.get_stack()
        self._undo.Enable(undo)
        self._redo.Enable(redo)

    def _on_undo(self, event):
        self._on_edit(None)
        if self._undo.IsEnabled():
            tab = self._memento.get_memento()
            print(tab)
            if tab:
                self._tabs.SetSelection(int(tab))
            self._update(check_for_errors=False)
            print("undo")

    def _on_redo(self, event):
        self._on_edit(None)
        if self._redo.IsEnabled():
            tab = self._memento.set_memento()
            self._tabs.SetSelection(int(tab))
            self._update(check_for_errors=False)
            print("redo")

    def _update(self, check_for_errors=True):
        if self._args is not None:
            if check_for_errors:
                ok = not self._check_errors()
            else:
                ok = True
            if ok:
                self._adv_tab.update_grid()
                self._adrs_tab.update_text_box()
                self._body_tab.update_tab()
                self._legs_tab.update_tab()
                self._arms_tab.update_tab()
                self._trunk_tab.update_tab()
                self._drivers_tab.update_tab()

    def _on_ok(self, event):
        self.Hide()
        self._hist.make_history_file_path()

        if self._connection is not None:
            load = self._load_model()
            if load == 2:
                self._error_loading = True
                self.Show()
                self._refresh_status()
            else:
                self._error_loading = False
                self._on_exit(None)
        else:
            self._error_loading = False
            self._on_exit(None)

    def _load_model(self, main_file=None):
        if main_file is None:
            load = self._connection.load_model()
        print(load)
        if load is None:
            self._warning(res.WARN_CLOSED)
            self._on_exit(None)
        elif not load:
            # self._warning(res.WARN_FAILED)
            title = "Model load failed"
            answer = self._interrogation(res.WARN_FAILED, title)
            if answer:
                self._on_undo(None)
                self._load_model(self._main_file)
            else:
                self._on_exit(None)
            # self._update()
        else:
            self._hist.save_last_ok_load()

        return load

    def _warning(self, message, caption="Error!"):
        dlg = wx.MessageDialog(self, message, caption, wx.OK | wx.ICON_ERROR)
        dlg.ShowModal()
        dlg.Destroy()

    def _on_load(self, event):
        self.Hide()
        load = self._load_model()
        if load == 2:
            self._error_loading = True
        else:
            self._error_loading = False
        self.Show()
        self._refresh_status()

    def _on_exit(self, event):
        self._closed = True
        position = self.GetPosition()
        self._appdata_file.ini_edit("Position", "Coordinates", position)

        tab = self._tabs.GetSelection()
        self._appdata_file.ini_edit("Tabs", "Selection", tab)

        self.Destroy()

    def _on_default(self, event):
        # self._adv_tab.reset_grid()
        if self._default_any_file is None:
            self._default_any_address = res.default_any_address
        else:
            self._default_any_address = self._default_any_file

        shutil.copy(self._default_any_address, self._any_file)

        self._update()
        self._memento.create_memento(0)

    def _on_choice(self, event):
        self._hist.apply_history_file(event.GetString())
        self._update()

    def _on_change(self, event):
        self._update()

    def _on_focus(self, event):
        if self._adrs_tab.got_address() and not self._closed:
            self._focus += 1
            # if divmod(self._focus, 2)[1] == 0:
            #     self._update()
            self._update()

    def _make_arguments(self, any_addr):
        arguments = Arguments()
        arguments.any_file = anyparse.ANY_Manipulator(any_addr, self._anyfolder)
        if self._anyfolder is not None:
            arguments.any_file.add_path_line()
        self._memento = memento.Memento(any_addr)
        arguments.memento = self._memento

        if self._bm_param_file is not None:
            f = self._bm_param_file
        else:
            f = res.bm_param_address
        x = bm_param_parser.Statements(f)
        arguments.options, arguments.options_dictionary = x.constants()
        arguments.statements, arguments.statements_dictionary = x.parameters()
        arguments.bm_param_file = f
        return arguments

    def _is_not_number(self, n):
        try:
            float(n)
            return False
        except ValueError:
            return True

    def _set_status(self, text_0, colour_0, text_1, colour_1):
        self._status["Colour0"] = colour_0
        self._status["Text0"] = text_0
        self._status["Colour1"] = colour_1
        self._status["Text1"] = text_1

    def _refresh_status(self):
        if self.statusbar is not None:
            self.statusbar.SetBackgroundColour(self._status["Colour0"])
            self.statusbar.SetStatusText(self._status["Text0"], 0)
            self.statusbar.SetStatusText(self._status["Text1"], 1)

    def _check_errors(self):
        errors_t = []
        errors_d = []
        error = True
        for parameter in self._args.statements:
            op = self._args.any_file.Get_parameter(parameter.name)
            not_string = parameter.valuetype != "string"
            if (
                op not in self._args.options_dictionary
                and op not in self._args.statements_dictionary
                and op != "None"
                and self._is_not_number(op)
                and not_string
            ):
                if op == "Multiple":
                    errors_d.append(parameter.name)
                else:
                    errors_t.append(parameter.name)

        if len(errors_t) and len(errors_d):
            error_list = errors_t
            error_list.extend(x for x in errors_d if x not in error_list)
            self._set_status(
                res.ERRSTAT, "RED", res.ERRTYPOS + str(error_list), "LIGHT_GRAY"
            )

        elif len(errors_t):
            self._set_status(
                res.ERRSTAT, "RED", res.ERRTYPOS + str(errors_t), "LIGHT_GRAY"
            )

        elif len(errors_d):
            self._set_status(
                res.ERRSTAT,
                "RED",
                res.ERRDUB + str(errors_d) + res.DEFAULT,
                "LIGHT_GRAY",
            )
            self._tabs.Disable()
            self._disabled = True
        elif self._error_loading:
            self._set_status(res.ERRSTAT, "RED", res.MODELFAIL, "LIGH_GRAY")
        elif not self._closed:
            # self.statusbar.SetStatusText(text, 1)
            self._set_status(res.OKSTAT, "LIGHT_GRAY", res.PROMPTCONF, "LIGHT_GRAY")
            error = False
            if self._disabled:
                self._tabs.Enable()
                self._disabled = False
        self._refresh_status()
        return error

    def create_remaining_tabs(self, any_file=None):

        while "Script" not in self._tabs.GetPageText(0):
            self._tabs.DeletePage(0)

        if any_file is None:
            any_file = self._adrs_tab.get_any_file_address()
        else:
            self._adrs_tab.set_any_file_address(any_file)

        self._hist = history.History(self._appdata_file, any_file)
        self._history.Enable()

        choices = self._hist.get_history_list()
        if len(choices) == 0:
            choices.append("No history available")
        else:
            choices = [choice.replace(".any", "") for choice in choices]
        self._history.AppendItems(choices)

        self._ok_button.Enable()
        self._default_button.Enable()
        if self._connected:
            self._load_button.Enable()
        self._any_file = any_file
        self._args = self._make_arguments(any_file)
        # if not self._check_errors():
        self._set_status(res.LOADSTAT, "YELLOW", res.LOADING, "LIGHT_GRAY")

        self._adv_tab = advtab.Advanced_Tab(self._tabs, self._args)
        self._body_tab = bodytab.BodyTab(self._tabs, self._args)
        self._legs_tab = legstab.LegsTab(self._tabs, self._args)
        self._arms_tab = armstab.ArmsTab(self._tabs, self._args)
        self._trunk_tab = trunktab.TrunkTab(self._tabs, self._args)
        self._drivers_tab = driverstab.DriversTab(self._tabs, self._args)

        self._tabs.InsertPage(0, self._adv_tab, res.ADV_TAB)
        self._tabs.InsertPage(0, self._drivers_tab, res.DRIVERS_TAB)
        self._tabs.InsertPage(0, self._trunk_tab, res.TRUNK_TAB)
        self._tabs.InsertPage(0, self._arms_tab, res.ARMS_TAB)
        self._tabs.InsertPage(0, self._legs_tab, res.LEGS_TAB)
        self._tabs.InsertPage(0, self._body_tab, res.BODY_TAB, True)

        self._adrs_tab.make_any_text(self._connected)

        self._set_status(res.OKSTAT, "LIGHT_GRAY", res.PROMPTCONF, "LIGHT_GRAY")
        self._create_tabs = True
        self._check_errors()
        self._undo.Enable(True)
        return 0
