"""
This module is used to define and create the 'Advanced' tab inside the plugin.
The 'Advanced' tab will be used to provide an overview on all the available
BM statements as well as a description for each BM statement and description
for the available options. This tab can also be considered the main tab of the
plugin, as the other tabs communicate with it and all the changes in the other
tabs will be noticed in the 'Advanced' tab as well and viceversa.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of February, 15:16, 2017

"""

import wx
import wx.grid

from bm_config import res


class Advanced_Tab(wx.Panel):
    def __init__(self, parent, args):

        """The Advanced tab is created by instantiating this class."""
        wx.Panel.__init__(self, parent)

        self._any_file = args.any_file
        self._parameters = args.statements
        self._param_dict = args.statements_dictionary
        self._options = args.options
        self._op_dict = args.options_dictionary
        self._memento = args.memento

        self._statementsno = 0

        self._grid = wx.grid.Grid(self, -1)
        self._grid.CreateGrid(0, 3)
        self._grid.DisableDragColSize()
        self._grid.DisableDragRowSize()
        if args.bm_param_file is not None:
            self._create_grid(args.bm_param_file)
        else:
            self._create_grid(res.bm_param_address)

        table_sizer = wx.BoxSizer(wx.VERTICAL)
        info_area_sizer = wx.BoxSizer(wx.VERTICAL)

        panel = wx.Panel(self, name="Description", size=(200, 60))

        self._description = wx.StaticText(panel, -1, res.DESCRIPTION)

        self._optgrid = wx.grid.Grid(self, -1, size=(200, 100))
        self._optgrid.CreateGrid(0, 1)
        self._optgrid.DisableDragColSize()
        self._optgrid.DisableDragRowSize()
        self._optgrid.SetColSize(0, 650)
        self._optgrid.HideRowLabels()
        self._optgrid.HideColLabels()

        self._grid.HideRowLabels()
        self._grid.SetColLabelValue(0, "No.")
        self._grid.SetColLabelValue(1, "Name")
        self._grid.SetColLabelValue(2, "Value")

        grid_id = self._grid.GetId()
        self.Bind(wx.grid.EVT_GRID_SELECT_CELL, self._on_cell_select, id=grid_id)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGED, self._on_cell_edit, id=grid_id)

        table_sizer.Add(self._grid, 1, wx.EXPAND, 3)
        table_sizer.Add(wx.StaticLine(self), 0, wx.ALL | wx.EXPAND, 3)
        info_area_sizer.Add(panel, 1, wx.EXPAND, 3)
        info_area_sizer.Add(self._optgrid, 0, wx.EXPAND, 3)
        table_sizer.Add(info_area_sizer, 0, wx.EXPAND, 3)

        self.SetSizer(table_sizer)
        self.Refresh()

    def update_grid(self):
        """Method for updating advanced grid following an external action."""
        for parameter in self._parameters:
            option = self._any_file.Get_parameter(parameter.name)
            if option != "None":
                self._update_cell(parameter.name, option)
            else:
                self._update_cell(parameter.name, parameter.default)

    def reset_grid(self):
        """Method for reseting values to default"""
        for parameter in self._parameters:
            self._update_cell(parameter.name, parameter.default)
            self._any_file.Default_param(parameter)

    def _create_grid(self, bm_param_address):
        grid = self._grid
        index = 0
        for parameter in self._parameters:
            if parameter.tag == "parameter":
                grid.AppendRows()

                grid.SetCellValue(index, 0, str(index + 1))
                grid.SetCellValue(index, 1, parameter.name)
                grid.SetReadOnly(index, 0, True)
                grid.SetReadOnly(index, 1, True)

                string_value = parameter.value
                if (
                    string_value is None
                    or string_value == "None"
                    or string_value == parameter.default
                ):

                    grid.SetCellValue(index, 2, parameter.default)
                    grid.SetCellBackgroundColour(index, 2, "WHITE")

                elif parameter.value != parameter.default:

                    grid.SetCellValue(index, 2, parameter.value)
                    grid.SetCellBackgroundColour(index, 2, "GREEN")

                if parameter.numofopt != 0:
                    choice_options = parameter.options
                    optops = wx.grid.GridCellChoiceEditor(choice_options, False)
                    grid.SetCellEditor(index, 2, optops)

                elif parameter.valuetype == "integer":

                    if parameter.default == "OFF" or parameter.default == "ON":
                        on_off = ["ON", "OFF"]
                        optionsonoff = wx.grid.GridCellChoiceEditor(on_off, False)
                        grid.SetCellEditor(index, 2, optionsonoff)

                    else:
                        optionsint = wx.grid.GridCellNumberEditor()
                        grid.SetCellEditor(index, 2, optionsint)

                elif parameter.valuetype == "float":

                    optionsfloat = wx.grid.GridCellFloatEditor(-1, 1)
                    grid.SetCellEditor(index, 2, optionsfloat)

                elif parameter.valuetype == "string":

                    optionsstring = wx.grid.GridCellTextEditor()
                    grid.SetCellEditor(index, 2, optionsstring)

            index += 1

        grid.SetColSize(0, 30)
        grid.SetColSize(1, 410)
        grid.SetColSize(2, 210)

        grid.HideRowLabels()

        grid.SetColLabelValue(0, "No.")
        grid.SetColLabelValue(1, "Name")
        grid.SetColLabelValue(2, "Value")
        self.update_grid()
        grid.ForceRefresh()
        self._grid = grid

    def _update_cell(self, parameter, option):
        index = 0
        for param in self._param_dict:
            if param == parameter:
                break
            else:
                index += 1
        self._grid.SetCellValue(index, 2, option)
        if option != self._param_dict[parameter].default:
            self._grid.SetCellBackgroundColour(index, 2, "YELLOW")
        else:
            self._grid.SetCellBackgroundColour(index, 2, "WHITE")

    def _on_cell_select(self, event):

        row = event.GetRow()
        label = self._parameters[row].description + "\nValue type: "
        label = label + self._parameters[row].valuetype
        label = label + "\nDefault Value: "
        label = label + self._parameters[row].default
        label = label + "\nAvailable selectable values ("
        label = label + str(self._parameters[row].numofopt) + "):"
        self._description.Label = label

        if self._parameters[row].numofopt:
            if self._optgrid.GetNumberRows():
                self._optgrid.DeleteRows(0, self._optgrid.GetNumberRows(), True)

            for index in range(0, self._parameters[row].numofopt):
                self._optgrid.AppendRows()
                self._optgrid.AppendRows()
                option = self._parameters[row].options[index]
                self._optgrid.SetCellValue(index * 2, 0, option)
                self._optgrid.SetCellBackgroundColour(index * 2, 0, (200, 200, 200))
                self._optgrid.SetReadOnly(index * 2, 0, True)
                description = 0

                for option in self._options:
                    if self._parameters[row].options[index] == option.name:
                        description = option.description
                        string_description = description
                        if (
                            not string_description == "None"
                            or string_description is not None
                        ):
                            self._optgrid.SetCellValue(
                                index * 2 + 1, 0, string_description
                            )
                            description = 1
                        else:
                            description = 1
                            self._optgrid.SetCellValue(index * 2 + 1, 0, res.NO_DESCR)
                        self._optgrid.SetReadOnly(index * 2 + 1, 0, True)
                        break
                if not description:
                    self._optgrid.SetCellValue(index * 2 + 1, 0, res.NO_DESCR)
        elif (
            self._parameters[row].default == "OFF"
            or self._parameters[row].default == "ON"
        ):
            if self._optgrid.GetNumberRows():
                self._optgrid.DeleteRows(0, self._optgrid.GetNumberRows(), True)

            label = self._parameters[row].description + "\nValue type: "
            label = label + self._parameters[row].valuetype
            label = label + "\nDefault Value: "
            label = label + self._parameters[row].default
            label = label + "\nAvailable selectable values (2):"

            self._description.Label = label

            self._optgrid.AppendRows()
            self._optgrid.SetCellValue(0, 0, "ON")
            self._optgrid.SetCellBackgroundColour(0, 0, (200, 200, 200))
            self._optgrid.AppendRows()
            self._optgrid.SetCellValue(1, 0, res.NO_DESCR)
            self._optgrid.AppendRows()
            self._optgrid.SetCellValue(2, 0, "OFF")
            self._optgrid.SetCellBackgroundColour(2, 0, (200, 200, 200))
            self._optgrid.AppendRows()
            self._optgrid.SetCellValue(3, 0, res.NO_DESCR)
        elif self._optgrid.GetNumberRows():
            self._optgrid.DeleteRows(0, self._optgrid.GetNumberRows(), True)

        self._optgrid.ForceRefresh()
        self._grid.ForceRefresh()

    def _on_cell_edit(self, event):

        row = event.GetRow()
        col = event.GetCol()
        cell = self._grid.GetCellValue(row, col)
        default = self._parameters[row].default
        if cell != default:
            self._grid.SetCellBackgroundColour(row, col, (100, 255, 255))
        else:
            self._grid.SetCellBackgroundColour(row, col, "WHITE")
        self._grid.ForceRefresh()
        self._any_file.Set_param(
            self._grid.GetCellValue(row, 1), self._grid.GetCellValue(row, col)
        )
        self._memento.create_memento(self.Parent.GetSelection())
