"""
This module is used to parse and modify '.any' files. An '.any' file will be
the output of the plugin for the AnyBody software to use and load the defined
BM statements. The statements inside the file are handled with the dictionary
data structure using the AnyPPConfig class from the anyppconfig.py file.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of February, 15:25, 2017

"""

import os

from bm_config import anyppconfig


class ANY_Manipulator(object):
    def __init__(self, address, anyfolder):
        """Class used for handling and editing .any files.

        address -- the path to the .any file
        stand_alone -- the mode in which the plugin is operating
        """
        if ".any" not in address:
            self._ws_path = address
            self._any_path = address + "\\ANY"

            self._outputname = "\\Model\\BodyModel.output.any"
            self._file_path = self._any_path + self._outputname
        else:
            self._file_path = address
        self._anyconfig = anyppconfig.AnyPPConfig(self._file_path)
        self._define = "#define "
        self._multi_occur = []
        self._anyfolder = anyfolder


    def add_path_line(self):
        """This method modifies the configuration file by adding path
        statements containing the absolute path to the file.
        A configuration file is created for each human model,
        it is therefore to be ensured that the path variable
        contains the name of the AnyFolder object in which the afferent
        human model is located.
        """
        with open(self._file_path) as f:
            content = f.readlines()

        anyfolder = self._anyfolder
        # The AnyFolder object in which the human model is located.
        #   EXAMPLE:
        #       H2

        file = os.path.basename(self._file_path)
        # The name of the configuration file is extracted from its
        #   absolute path.
        # EXAMPLE:
        #    H2_BodyModelConfiguration.any

        existing_path_line = (
            "#path __BM_PLUGIN_FILE" + anyfolder + '__ "' + file + '"\n'
        )
        # The configuration file should contain this path declaration.
        # EXAMPLE:
        #   #path __BM_PLUGIN_FILE__ "BodyModelConfiguration.any"

        path_line = (
            existing_path_line
            + "#define BM_CONFIG_PLUGIN __BM_PLUGIN_FILE"
            + anyfolder
            + "__\n\n"
        )
        # If no path declaration is found, this path declaration is added:
        # #path __BM_PLUGIN_FILE__ "BodyModelConfiguration.any"
        # #define BM_CONFIG_PLUGIN __BM_PLUGIN_FILE__

        for line in content:
            if existing_path_line in line:
                break
        else:
            content.insert(0, path_line)

        f = open(self._file_path, "w")
        for line in content:
            f.write(line)
        f.close()

    def ANY_Create(self):
        """The any file is created if does not exist"""

        if not os.path.exists(self._ws_path):
            os.mkdir(self._ws_path)
        if not os.path.exists(self._any_path):
            os.mkdir(self._any_path)
        if not os.path.isfile(self._file_path):

            file = open(self._file_path, "w")
            file.close()

    def ANY_Edit(self, parameters):
        """The paramters in the .any file are updated with new values from the
        plugin. This method is used if it is desired for the statements in the
        .any file to be deleted if they have the default value

        parameters -- list containing parameter objects
        """

        for parameter in parameters:
            if (
                parameter.value == parameter.default
                and parameter.name in self._anyconfig
            ):
                del self._anyconfig[parameter.name]
            elif not parameter.value == "None" and parameter.value != parameter.default:
                self._anyconfig[parameter.name] = parameter.value

    def ANY_Read(self, parameters):
        """The parameter values inside the .any file are extracted

        paramters -- list containing parameter objects in which the values will
        be stored
        """

        for parameter in parameters:
            try:
                parameter.value = self._anyconfig[parameter.name]
            except KeyError:
                pass
        return parameters

    def ANY_Edit_v2(self, parameters):
        """The paramters in the .any file are updated with new values from the
        plugin.

        parameters -- list containing parameter objects
        """
        for parameter in parameters:
            if (
                parameter.value != parameter.default
                or parameter.name in self._anyconfig
            ):
                self._anyconfig[parameter.name] = parameter.value

    def Get_duplicates(self, parameters):
        errors = []
        for parameter in parameters:
            try:
                self._anyconfig[parameter.name]
            except ValueError:
                errors.append(parameter.name)
            except KeyError:
                pass

        return errors

    def Get_parameter(self, parameter):
        """Return the value of a certain parameter

        parameter -- Name of the parameter (not nicename)
        """
        try:
            return self._anyconfig[parameter]
        except KeyError:
            return "None"
        except ValueError:
            return "Multiple"

    def Get_value(self, parameter):
        try:
            return self._anyconfig[parameter.name]
        except KeyError:
            return parameter.default
        except ValueError:
            self._multi_occur.append(parameter.name)
            return parameter.default

    def Get_multi_errors(self):
        return self._multi_occur

    def Clear_errors(self):
        self._multi_occur.clear()

    def Set_parameter(self, parameter, option):
        if (
            option not in parameter.default
            or self.Get_value(parameter) not in parameter.default
        ):
            self._anyconfig[parameter.name] = option

    def Set_param(self, parameter, option):
        self._anyconfig[parameter] = option

    def Default_param(self, parameter):
        if self.Get_parameter(parameter.name) != "None":
            self.Set_parameter(parameter, parameter.default)

    def Remove_parameter(self, paramter):
        del self._anyconfig[paramter.name]
