import os
import re
import collections
from typing import NamedTuple
from typing import Tuple, List

MATCH_TEMPL = (
    r"^(?P<prefix>[^\S\n]*"  # match whitespace (excl. newline) at start of
    # line
    r"(?P<type>#define|#path)"  # match #define/#path prefix
    r"[^\S\n]+)"  # match whitespace (excl. newline)
    r"(?P<key>{key})"  # match the variable name (key)
    r"(?P<space>[^\S\n]+)"  # match space between variable name and value
    r'(?P<value>(?:".*?"|[^"]*?)*?(?=[^\S\n]*(?://|$|/\*)))'  # non greedy
    # match for switch value
    r"(?P<postfix>.*)$"  # match postfix e.g. whitespaces or comments
)

# Pattern for updateting (replace) a value
REPLACE_TEMPL = r"\g<prefix>\g<key>\g<space>{val}\g<postfix>"

# Pattern for updateting (replace) a value
DELETE_TEMPL = r"\g<postfix>"

# Create compiled regex object that match all define/path statements
RE_PARSE_ALL = re.compile(MATCH_TEMPL.format(key=r"\w+"), re.M)

SwitchTuple = NamedTuple("SwitchTuple", [("type", str), ("key", str), ("value", str)])


class AnyPPConfig(collections.abc.MutableMapping):
    """
    Implements a dictionary interface to an underlying anyscript file.
    The defines/path statements of anyscript file can be manipulated by
    changing the mapping.

    anyconfig = AnyPPConfig('BodyModelConfig.any')
    # Change BM_TRUNK_NECK to off or add it if it doesn't exist
    anyconfig['BM_TRUNK_NECK'] = 'OFF'

    New statements are added to the buttom of the file.
    All new staments added through the dict interface are treated as
    #define's, unless stateted explicitly e.g.:
    anyconfig['#path AMMR'] = '"../ammr/"'

    Attempts to set or read values which are defined multiple time in the
    anyscript file will raise ValueError

    """

    def __init__(self, filename: str, *args, **kwargs) -> None:
        self._filename = filename
        self.update(*args, **kwargs)

    # def _file_changed(self):
    #     stamp = 0
    #     attempts = 10
    #     while attempts < 10:
    #         stamp, old_stamp = os.stat(self._filename).st_mtime, stamp
    #         if stamp == old_stamp:
    #             yield False
    #         else:
    #             yield True
    #     raise IOError('Unable to write to an unchanged file')

    def read_file(self) -> str:
        """return the content of the file"""
        with open(self._filename, encoding="utf8") as f:
            return f.read()

    @staticmethod
    def _parse(content) -> List[SwitchTuple]:
        """
        Parses a string containing an anyscript file for define/path
        statements.
        Creates a list of named tuples with entries for type, key, and value of
        the define/path statements.
        """
        mlist = RE_PARSE_ALL.findall(content)
        matches = []  # tuple: List[Tuple[key, VarInfo]]
        if mlist:
            for match in mlist:
                stuple = SwitchTuple(type=match[1], key=match[2], value=match[4])
                matches.append(stuple)
        return matches

    def _split_key(self, key: str) -> Tuple[str, str]:
        """
        Splits the keyname into a keytype (#define/#path) and keyname, and
        returns
        a (keytype, keyname) tuple. If the key name is not qualified then
        return
        (None, keyname)
        """
        if key.startswith("#"):
            ktype, key = key.split()
        else:
            ktype, key = None, key
        return ktype, key

    def __setitem__(self, key: str, value: str) -> None:
        keytype, key = self._split_key(key)
        self._check_value(value)
        content = self.read_file()
        matches = self._parse(content)
        self._check_multiple_entries(key, matches)
        kmap = {elem.key: elem for elem in matches}
        if key in kmap:
            if keytype is not None and keytype != kmap[key].type:
                raise KeyError(
                    "The type (define/path) does not match"
                    " the variable already in the file"
                )
            pattern = MATCH_TEMPL.format(key=key)
            repl = REPLACE_TEMPL.format(val=value)
            out, n = re.subn(pattern, repl, content, flags=re.M)
            if n != 1:
                ValueError("Regex errro. Could not update value")
        else:
            if not keytype:
                keytype = "#define"
            out = "{}\n{} {} {}".format(content, keytype, key, value)
        with open(self._filename, "w") as f:
            f.write(out)

    def _check_value(self, value: str) -> None:
        """Check that the value given is valid"""
        if not isinstance(value, str):
            raise ValueError("Values must be Strings")
        if value.startswith('"') or value.endswith('"'):
            if value[0] != value[-1]:
                raise ValueError("Unbalanced quotes")

    def __getitem__(self, key: str) -> str:
        ktype, key = self._split_key(key)
        matches = self._parse(self.read_file())
        self._check_multiple_entries(key, matches)
        kmap = {elem.key: elem for elem in matches}
        if key in kmap and (ktype is None or ktype == kmap[key].type):
            return kmap[key].value
        else:
            self._raise_keyerror(ktype, key)

    def _check_multiple_entries(self, key: str, matches: List[SwitchTuple]) -> None:
        """
        Checks that the names are not defined multiple times in the file.
        Raises ValueError otherwise.
        """
        if matches:
            _, key = self._split_key(key)
            keylist = [e.key for e in matches]
            if keylist.count(key) > 1:
                raise ValueError(
                    key + " is defined multiple times in the file. "
                    "Only variables defined once can be modified"
                )

    def get_type(self, key: str) -> str:
        """Return the type of switch statement (define/path)
        if it is in the file."""

        ktype, key = self._split_key(key)
        matches = self._parse(self.read_file())
        self._check_multiple_entries(key, matches)
        kmap = {elem.key: elem for elem in matches}
        if key in kmap:
            return kmap[key].type
        else:
            self._raise_keyerror(None, key)

    def _raise_keyerror(self, ktype: str, key: str) -> None:
        prefix = ktype + " " if ktype else ""
        raise KeyError(prefix + key)

    def __delitem__(self, key):
        ex = r"(\s*|\n*)#define\s*" + key + r"\s*\w*"
        to_find = re.compile(ex)
        ktype, key = self._split_key(key)
        content = self.read_file()
        matches = self._parse(content)
        self._check_multiple_entries(key, matches)
        kmap = {elem.key: elem for elem in matches}
        if key in kmap and (ktype is None or ktype == kmap[key].type):
            pattern = MATCH_TEMPL.format(key=key)
            out, no_replace = re.subn(pattern, DELETE_TEMPL, content, flags=re.M)
            if no_replace > 1:
                raise ValueError(
                    key + "name is defined multiple times in the file. "
                    "Only variables defined once can be modified"
                )
            elif no_replace == 0:
                raise ValueError(
                    key + "could not be matched in the file. Either the"
                    " file was changed, "
                    "or some other error occured."
                )
        else:
            self._raise_keyerror(ktype, key)
        match = to_find.search(content)
        match = match.group()
        head, tail = content.split(match)

        out = head + tail

        with open(self._filename, "w") as f:
            f.write(out)

    def __iter__(self):
        stamp = os.stat(self._filename).st_mtime
        matches = self._parse(self.read_file())
        counts = collections.Counter([elem.key for elem in matches])
        kmap = {elem.key: elem for elem in matches if counts[elem.key] == 1}
        for k in kmap:
            if stamp != os.stat(self._filename).st_mtime:
                raise IOError("AnyScript file changed while iterating over it")
            yield k

    def __len__(self):
        """Returns number of define/path statements in the file, which only
        occurs ones in the file"""
        matches = self._parse(self.read_file())
        counts = collections.Counter([elem.key for elem in matches])
        return [counts.values()].count(1)

    def __str__(self):
        """returns simple dict representation of the mapping"""
        matches = self._parse(self.read_file())
        counts = collections.Counter([elem.key for elem in matches])
        kmap = {elem.key: elem for elem in matches if counts[elem.key] == 1}
        return kmap.__str__()

    def __repr__(self):
        """echoes class, id, & reproducible representation in the REPL"""
        matches = self._parse(self.read_file())
        counts = collections.Counter([elem.key for elem in matches])
        kmap = {elem.key: elem for elem in matches if counts[elem.key] == 1}
        return 'AnyPPConfig("{}"),{}'.format(self._filename, kmap.__repr__())
