"""
This module is used to define and create the 'Body' tab inside the plugin.
Inside this tab the user can choose what limbs will use and the type of the
limbs.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Created on 7th of February, 15:16, 2017

"""

import wx
from bm_config import images


class BodyTab(wx.Panel):
    def __init__(self, parent, args):
        """The Body tab is created by instantiating this class."""
        wx.Panel.__init__(self, parent)

        self._any_file = args.any_file
        self._statements = args.statements
        self._pr_dict = args.statements_dictionary
        self._options = args.options
        self._op_dict = args.options_dictionary
        self._memento = args.memento
        self._current_image = None

        self._checkboxes = {}
        self._choices = {}
        self._texts = {}

        self._imgs = images.Images(self, "Body_Tab", 500, 435)

        self._make_gui_elements()
        self.update_tab()
        self._set_image()

    def update_tab(self):
        """Method for updating GUI elements following an external action."""
        for key in self._choices:
            choice = self._choices[key]
            name = choice.Name
            op = self._any_file.Get_parameter(name)
            if op == "None":
                choice.SetStringSelection(self._pr_dict[name].default)
            else:
                choice.SetStringSelection(self._op_dict[op].nicename)

        for key in self._checkboxes:
            chbox = self._checkboxes[key]
            op = self._any_file.Get_parameter(chbox.Name)
            if op == "None":
                default = self._pr_dict[chbox.Name].default
                if any(key in default for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)
            else:
                if any(key in op for key in ["NONE", "OFF"]):
                    switch = False
                else:
                    switch = True
                chbox.SetValue(switch)

        self._set_image()

    def _make_gui_elements(self):
        """The GUI elements inside the Body tab are created"""

        #       Checkboxes:
        self._create_checkbox((170, 25), "BM_ARM_LEFT")
        self._create_checkbox((25, 25), "BM_ARM_RIGHT")
        self._create_checkbox((25, 50), "BM_ARM_DETAILED_HAND")
        self._create_checkbox((170, 120), "BM_LEG_LEFT")
        self._create_checkbox((25, 120), "BM_LEG_RIGHT")

        #       Statics texts:
        name = "type_leg"
        text = wx.StaticText(self, pos=(25, 145), name=name, label="Type:")
        self._texts[name] = text

        #       Droplists:
        name = "BM_LEG_MODEL"
        choices = self._pr_dict[name].options
        choices = [x for x in choices if "OFF" not in x]
        choices = [self._op_dict[x].nicename for x in choices]
        choice = wx.Choice(self, pos=(65, 141), choices=choices, name=name)
        value = self._any_file.Get_value(self._pr_dict[name])
        value = self._op_dict[value].nicename
        choice.SetStringSelection(value)
        self.Bind(wx.EVT_CHOICE, self._on_choice, id=choice.GetId())
        self._choices[name] = choice

    def _create_checkbox(self, pos, name):
        label = self._pr_dict[name].nicename
        chbox = wx.CheckBox(self, pos=pos, label=label, name=name, style=wx.CHK_3STATE)
        value = self._any_file.Get_value(self._pr_dict[name])
        if any(key in value for key in ["NONE", "OFF"]):
            switch = False
        else:
            switch = True
        chbox.SetValue(switch)
        self.Bind(wx.EVT_CHECKBOX, self._on_check_box, id=chbox.GetId())
        self._checkboxes[name] = chbox

    def _update_any(self):

        for key in self._checkboxes:
            chbox = self._checkboxes[key]
            for statement in self._statements:
                for chbox_name in chbox.Name.split():
                    if statement.name == chbox_name:
                        value = chbox.GetValue()
                        for option in statement.options:
                            if value:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "ON")
                                    statement.value = True
                                    break
                                elif "NONE" not in option and "OFF" not in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = True
                                    break
                            else:
                                if option is None:
                                    self._any_file.Set_parameter(statement, "OFF")
                                    statement.value = False
                                    break
                                elif "NONE" in option or "OFF" in option:
                                    self._any_file.Set_parameter(statement, option)
                                    statement.value = False
                                    break

        for key in self._choices:
            choice = self._choices[key]
            for statement in self._statements:
                for choice_name in choice.Name.split():
                    if statement.name == choice_name:
                        value = choice.GetStringSelection()
                        for option in self._options:
                            if option.nicename == value and statement.value:
                                self._any_file.Set_parameter(statement, option.name)
                                break
                        break

        self._memento.create_memento(self.Parent.GetSelection())
        self.update_tab()

    def _set_image(self):
        """Updates the image of the body model in the tab by loading the image
        with the name corresponding to the current settings in the tab. The
        method is used to update the values of the GUI elements as well, if
        values from other tabs or from the advanced tab grid were changed.
        """

        if (
            not self._checkboxes["BM_ARM_RIGHT"].GetValue()
            and not self._checkboxes["BM_ARM_LEFT"].GetValue()
        ):
            self._checkboxes["BM_ARM_DETAILED_HAND"].Disable()
        else:
            self._checkboxes["BM_ARM_DETAILED_HAND"].Enable()

        if (
            not self._checkboxes["BM_LEG_RIGHT"].GetValue()
            and not self._checkboxes["BM_LEG_LEFT"].GetValue()
        ):
            self._choices["BM_LEG_MODEL"].Disable()
        else:
            self._choices["BM_LEG_MODEL"].Enable()

        if self._checkboxes["BM_ARM_RIGHT"].GetValue():
            val = self._checkboxes["BM_ARM_DETAILED_HAND"].GetValue() + 1
            name = str(val)
        else:
            name = "0"

        if self._checkboxes["BM_ARM_LEFT"].GetValue():
            val = self._checkboxes["BM_ARM_DETAILED_HAND"].GetValue() + 1
            name = name + str(val)
        else:
            name = name + "0"

        if self._checkboxes["BM_LEG_RIGHT"].GetValue():
            val = self._choices["BM_LEG_MODEL"].GetStringSelection()
            for option in self._op_dict.values():
                if option.nicename == val:
                    break
            else:
                option = self._op_dict["_LEG_MODEL_TLEM_"]
            val = int(option.default)
            if "v2" in self._choices["BM_LEG_MODEL"].GetStringSelection():
                val = val - 1

            name = name + str(val)
        else:
            name = name + "0"

        if self._checkboxes["BM_LEG_LEFT"].GetValue():
            val = self._choices["BM_LEG_MODEL"].GetStringSelection()
            for option in self._op_dict.values():
                if option.nicename == val:
                    break
            else:
                option = self._op_dict["_LEG_MODEL_TLEM_"]
            val = int(option.default)
            if "v2" in self._choices["BM_LEG_MODEL"].GetStringSelection():
                val = val - 1
            name = name + str(val)
        else:
            name = name + "0"
        if name != self._current_image:
            self._imgs.set_image(name)
            self._current_image = name

    # Events: #####################################################################

    def _on_check_box(self, event):
        self._update_any()

    def _on_choice(self, event):
        self._update_any()
