"""
This module is used to parse and modify '.ini' files. An '.ini' file is used
to keep information like the position of the frame and the addresses of the
AnyBody workspaces on the hard-drive in order to be used when the plugin will
be executed next time.

Author: Ioan-Matei Sarivan @ AnyBody Technology A/S
E-mail: ms@anybodytech.com

Cretaed on 7th of January, 15:16, 2017

"""

import os

from configobj import ConfigObj


class INIParse(object):
    def __init__(self, path, ini_file):
        """Class used for handling and editing .ini files.

        path -- the path to the .ini file
        ini_file -- name of the ini file
        """

        self._parent_path = path
        self._path = path + r"\INI"
        self._file_path = self._path + "\\" + ini_file + ".ini"

        self._exists = self._check_existance()

    def exist(self):
        """Returns True if the .ini file exists"""
        return self._exists

    def _check_existance(self):
        """Check if there is an .ini file with the name specified at the given
        path"""
        exists = True

        if not os.path.exists(self._path):
            os.makedirs(self._path)
            exists = False

        if not os.path.isfile(self._file_path):
            file = open(self._file_path, "w")
            file.close()
            exists = False

        if not os.stat(self._file_path).st_size:
            exists = False

        return exists

    def ini_edit(self, section, parameter, value):
        """Edits the value of a certain parameter in the given section

        section -- name of the section in which the parameter is found
        parameter -- name of the parameter
        value -- the new value for the parameter
        """
        if self._check_existance():
            ini = ConfigObj(self._file_path)
            if section in ini:
                ini[section][parameter] = value
            else:
                ini[section] = {}
                ini[section][parameter] = value

            ini.write()

        else:

            ini = ConfigObj(self._file_path)
            ini[section] = {}
            ini[section][parameter] = value

            ini.write()

            self._exists = True

    def ini_read(self, section, parameter):
        """Returns the value of the parameter in the given section

        section -- name of the section in which the parameter is found
        paramater -- the name of the parameter of which value has to be known
        """

        if self._exists:
            ini = ConfigObj(self._file_path)
            if section in ini:
                if parameter in ini[section].keys():
                    return ini[section][parameter]
                else:
                    return None
            else:
                return None

    def check_section_keys(self, section, key):
        if self._exists:
            ini = ConfigObj(self._file_path)
            if key in ini[section].keys():
                return True
            else:
                return False

    def ini_get_subkeys(self, section):
        if self._exists:
            ini = ConfigObj(self._file_path)
            if section in ini.keys():
                return ini[section].keys()
            else:
                return None

    def get_working_address(self):
        """Returns the absolute path of the .ini file"""
        return self._file_path

    def get_parent_path(self):
        """Returns the path of the .ini file"""
        return self._parent_path
